=== modified file 'NEWS'
--- NEWS	2006-12-21 19:32:25 +0000
+++ NEWS	2007-01-17 01:49:23 +0000
@@ -2,6 +2,19 @@
 
   IMPROVEMENTS:
 
+  INTERNALS:
+
+  BUGFIXES:
+
+  TESTING:
+
+    * New ``--first`` option to ``bzr selftest`` to run specified tests
+      before the rest of the suite.  (Martin Pool)
+
+bzr 0.14rc1  2007-01-16
+
+  IMPROVEMENTS:
+
     * New connection: ``bzr+http://`` which supports tunnelling the smart
       protocol over an HTTP connection. If writing is enabled on the bzr
       server, then you can write over the http connection.
@@ -56,6 +69,8 @@
     * ``bzr merge`` now has an option ``--pull`` to fall back to pull if
       local is fully merged into remote. (Jan Hudec)
 
+    * ``bzr help formats`` describes available directory formats. (Aaron Bentley)
+
   INTERNALS:
 
     * A few tweaks directly to ``fileids_affected_by_revision_ids`` to
@@ -77,6 +92,11 @@
     * Single-letter short options are no longer globally declared.  (Martin
       Pool)
 
+    * Before using detected user/terminal encoding bzr should check
+      that Python has corresponding codec. (Alexander Belchenko)
+
+    * Formats for end-user selection are provided via a FormatRegistry (Aaron Bentley)
+
   BUG FIXES:
 
     * ``bzr missing --verbose`` was showing adds/removals in the wrong
@@ -118,6 +138,9 @@
     * Mention the revisionspec topic for the revision option help.
       (Wouter van Heyst, #31663)
 
+    * Allow plugins import from zip archives.
+      (Alexander Belchenko, #68124)
+
 
 bzr 0.13  2006-12-05
     

=== modified file 'bzr'
--- bzr	2006-11-27 23:11:54 +0000
+++ bzr	2007-01-16 15:56:53 +0000
@@ -76,7 +76,7 @@
 import bzrlib.commands
 import bzrlib.trace
 
-if bzrlib.version_info[:3] != (0, 14, 0):
+if bzrlib.version_info[:3] != (0, 15, 0):
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"
             "bzrlib from %s is version %r\n"

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2006-12-01 19:41:16 +0000
+++ bzrlib/__init__.py	2007-01-16 15:56:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 user_encoding = get_user_encoding()
 
 
-__copyright__ = "Copyright 2005, 2006 Canonical Ltd."
+__copyright__ = "Copyright 2005, 2006, 2007 Canonical Ltd."
 
 # same format as sys.version_info: "A tuple containing the five components of
 # the version number: major, minor, micro, releaselevel, and serial. All
@@ -35,7 +35,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (0, 14, 0, 'dev', 0)
+version_info = (0, 15, 0, 'dev', 0)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-01-02 20:15:07 +0000
+++ bzrlib/builtins.py	2007-01-17 14:33:15 +0000
@@ -95,25 +95,15 @@
 
 def get_format_type(typestring):
     """Parse and return a format specifier."""
-    if typestring == "weave":
-        return bzrdir.BzrDirFormat6()
+    # Have to use BzrDirMetaFormat1 directly, so that
+    # RepositoryFormat.set_default_format works
     if typestring == "default":
         return bzrdir.BzrDirMetaFormat1()
-    if typestring == "metaweave":
-        format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormat7()
-        return format
-    if typestring == "knit":
-        format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit1()
-        return format
-    if typestring == "experimental-knit2":
-        format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit2()
-        return format
-    msg = "Unknown bzr format %s. Current formats are: default, knit,\n" \
-          "metaweave and weave" % typestring
-    raise errors.BzrCommandError(msg)
+    try:
+        return bzrdir.format_registry.make_bzrdir(typestring)
+    except KeyError:
+        msg = 'Unknown bzr format "%s". See "bzr help formats".' % typestring
+        raise errors.BzrCommandError(msg)
 
 
 # TODO: Make sure no commands unconditionally use the working directory as a
@@ -844,12 +834,7 @@
                                              % to_location)
             else:
                 raise
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            source.create_checkout(to_location, revision_id, lightweight)
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        source.create_checkout(to_location, revision_id, lightweight)
 
 
 class cmd_renames(Command):
@@ -2065,22 +2050,29 @@
 class cmd_selftest(Command):
     """Run internal test suite.
     
-    This creates temporary test directories in the working directory,
-    but not existing data is affected.  These directories are deleted
-    if the tests pass, or left behind to help in debugging if they
-    fail and --keep-output is specified.
+    This creates temporary test directories in the working directory, but not
+    existing data is affected.  These directories are deleted if the tests
+    pass, or left behind to help in debugging if they fail and --keep-output
+    is specified.
     
-    If arguments are given, they are regular expressions that say
-    which tests should run.
+    If arguments are given, they are regular expressions that say which tests
+    should run.  Tests matching any expression are run, and other tests are
+    not run.
+
+    Alternatively if --first is given, matching tests are run first and then
+    all other tests are run.  This is useful if you have been working in a
+    particular area, but want to make sure nothing else was broken.
 
     If the global option '--no-plugins' is given, plugins are not loaded
     before running the selftests.  This has two effects: features provided or
     modified by plugins will not be tested, and tests provided by plugins will
     not be run.
 
-    examples:
+    examples::
         bzr selftest ignore
+            run only tests relating to 'ignore'
         bzr --no-plugins selftest -v
+            disable plugins and list tests as they're run
     """
     # TODO: --list should give a list of all available tests
 
@@ -2121,12 +2113,16 @@
                      Option('clean-output',
                             help='clean temporary tests directories'
                                  ' without running tests'),
+                     Option('first',
+                            help='run all tests, but run specified tests first',
+                            )
                      ]
     encoding_type = 'replace'
 
     def run(self, testspecs_list=None, verbose=None, one=False,
             keep_output=False, transport=None, benchmark=None,
-            lsprof_timed=None, cache_dir=None, clean_output=False):
+            lsprof_timed=None, cache_dir=None, clean_output=False,
+            first=False):
         import bzrlib.ui
         from bzrlib.tests import selftest
         import bzrlib.benchmarks as benchmarks
@@ -2165,7 +2161,9 @@
                               transport=transport,
                               test_suite_factory=test_suite_factory,
                               lsprof_timed=lsprof_timed,
-                              bench_history=benchfile)
+                              bench_history=benchfile,
+                              matching_tests_first=first,
+                              )
         finally:
             if benchfile is not None:
                 benchfile.close()

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2007-01-11 14:26:18 +0000
+++ bzrlib/bzrdir.py	2007-01-17 14:33:15 +0000
@@ -30,6 +30,7 @@
 
 from cStringIO import StringIO
 import os
+import textwrap
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -42,8 +43,10 @@
     errors,
     lockable_files,
     lockdir,
+    registry,
     revision as _mod_revision,
     repository as _mod_repository,
+    symbol_versioning,
     urlutils,
     xml4,
     xml5,
@@ -202,18 +205,18 @@
 
     # TODO: Should take a Transport
     @classmethod
-    def create(cls, base):
+    def create(cls, base, format=None):
         """Create a new BzrDir at the url 'base'.
         
         This will call the current default formats initialize with base
         as the only parameter.
 
-        If you need a specific format, consider creating an instance
-        of that and calling initialize().
+        :param format: If supplied, the format of branch to create.  If not
+            supplied, the default is used.
         """
         if cls is not BzrDir:
-            raise AssertionError("BzrDir.create always creates the default format, "
-                    "not one of %r" % cls)
+            raise AssertionError("BzrDir.create always creates the default"
+                " format, not one of %r" % cls)
         head, tail = urlutils.split(base)
         if tail and tail != '.':
             t = get_transport(head)
@@ -221,7 +224,9 @@
                 t.mkdir(tail)
             except errors.FileExists:
                 pass
-        return BzrDirFormat.get_default_format().initialize(safe_unicode(base))
+        if format is None:
+            format = BzrDirFormat.get_default_format()
+        return format.initialize(safe_unicode(base))
 
     def create_branch(self):
         """Create a branch in this BzrDir.
@@ -232,7 +237,7 @@
         raise NotImplementedError(self.create_branch)
 
     @staticmethod
-    def create_branch_and_repo(base, force_new_repo=False):
+    def create_branch_and_repo(base, force_new_repo=False, format=None):
         """Create a new BzrDir, Branch and Repository at the url 'base'.
 
         This will use the current default BzrDirFormat, and use whatever 
@@ -245,7 +250,7 @@
         :param base: The URL to create the branch at.
         :param force_new_repo: If True a new repository is always created.
         """
-        bzrdir = BzrDir.create(base)
+        bzrdir = BzrDir.create(base, format)
         bzrdir._find_or_create_repository(force_new_repo)
         return bzrdir.create_branch()
 
@@ -289,10 +294,7 @@
             t = get_transport(safe_unicode(base))
             if not isinstance(t, LocalTransport):
                 raise errors.NotLocalUrl(base)
-        if format is None:
-            bzrdir = BzrDir.create(base)
-        else:
-            bzrdir = format.initialize(base)
+        bzrdir = BzrDir.create(base, format)
         repo = bzrdir._find_or_create_repository(force_new_repo)
         result = bzrdir.create_branch()
         if force_new_tree or (repo.make_working_trees() and 
@@ -304,11 +306,12 @@
         return result
         
     @staticmethod
-    def create_repository(base, shared=False):
+    def create_repository(base, shared=False, format=None):
         """Create a new BzrDir and Repository at the url 'base'.
 
-        This will use the current default BzrDirFormat, and use whatever 
-        repository format that that uses for bzrdirformat.create_repository.
+        If no format is supplied, this will default to the current default
+        BzrDirFormat by default, and use whatever repository format that that
+        uses for bzrdirformat.create_repository.
 
         :param shared: Create a shared repository rather than a standalone
                        repository.
@@ -318,11 +321,11 @@
         it should take no parameters and construct whatever repository format
         that child class desires.
         """
-        bzrdir = BzrDir.create(base)
+        bzrdir = BzrDir.create(base, format)
         return bzrdir.create_repository(shared)
 
     @staticmethod
-    def create_standalone_workingtree(base):
+    def create_standalone_workingtree(base, format=None):
         """Create a new BzrDir, WorkingTree, Branch and Repository at 'base'.
 
         'base' must be a local path or a file:// url.
@@ -337,7 +340,8 @@
         if not isinstance(t, LocalTransport):
             raise errors.NotLocalUrl(base)
         bzrdir = BzrDir.create_branch_and_repo(safe_unicode(base),
-                                               force_new_repo=True).bzrdir
+                                               force_new_repo=True,
+                                               format=format).bzrdir
         return bzrdir.create_workingtree()
 
     def create_workingtree(self, revision_id=None):
@@ -1226,7 +1230,13 @@
         klass._control_formats.append(format)
 
     @classmethod
+    @symbol_versioning.deprecated_method(symbol_versioning.zero_fourteen)
     def set_default_format(klass, format):
+        klass._set_default_format(format)
+
+    @classmethod
+    def _set_default_format(klass, format):
+        """Set default format (for testing behavior of defaults only)"""
         klass._default_format = format
 
     def __str__(self):
@@ -1470,7 +1480,7 @@
 BzrDirFormat.register_format(BzrDirFormat6())
 __default_format = BzrDirMetaFormat1()
 BzrDirFormat.register_format(__default_format)
-BzrDirFormat.set_default_format(__default_format)
+BzrDirFormat._default_format = __default_format
 
 
 class BzrDirTestProviderAdapter(object):
@@ -1961,3 +1971,133 @@
                 converter = CopyConverter(self.target_format.repository_format)
                 converter.convert(repo, pb)
         return to_convert
+
+
+class BzrDirFormatInfo(object):
+
+    def __init__(self, native, deprecated):
+        self.deprecated = deprecated
+        self.native = native
+
+
+class BzrDirFormatRegistry(registry.Registry):
+    """Registry of user-selectable BzrDir subformats.
+    
+    Differs from BzrDirFormat._control_formats in that it provides sub-formats,
+    e.g. BzrDirMeta1 with weave repository.  Also, it's more user-oriented.
+    """
+
+    def register_metadir(self, key, repo, help, native=True, deprecated=False):
+        """Register a metadir subformat.
+        
+        repo is the repository format name as a string.
+        """
+        # This should be expanded to support setting WorkingTree and Branch
+        # formats, once BzrDirMetaFormat1 supports that.
+        def helper():
+            import bzrlib.repository
+            repo_format = getattr(bzrlib.repository, repo)
+            bd = BzrDirMetaFormat1()
+            bd.repository_format = repo_format()
+            return bd
+        self.register(key, helper, help, native, deprecated)
+
+    def register(self, key, factory, help, native=True, deprecated=False):
+        """Register a BzrDirFormat factory.
+        
+        The factory must be a callable that takes one parameter: the key.
+        It must produce an instance of the BzrDirFormat when called.
+
+        This function mainly exists to prevent the info object from being
+        supplied directly.
+        """
+        registry.Registry.register(self, key, factory, help, 
+            BzrDirFormatInfo(native, deprecated))
+
+    def register_lazy(self, key, module_name, member_name, help, native=True,
+                      deprecated=False):
+        registry.Registry.register_lazy(self, key, module_name, member_name, 
+            help, BzrDirFormatInfo(native, deprecated))
+
+    def set_default(self, key):
+        """Set the 'default' key to be a clone of the supplied key.
+        
+        This method must be called once and only once.
+        """
+        registry.Registry.register(self, 'default', self.get(key), 
+            self.get_help(key), info=self.get_info(key))
+
+    def set_default_repository(self, key):
+        """Set the FormatRegistry default and Repository default.
+        
+        This is a transitional method while Repository.set_default_format
+        is deprecated.
+        """
+        if 'default' in self:
+            self.remove('default')
+        self.set_default(key)
+        format = self.get('default')()
+        assert isinstance(format, BzrDirMetaFormat1)
+        from bzrlib import repository
+        repository.RepositoryFormat._set_default_format(
+            format.repository_format)
+
+    def make_bzrdir(self, key):
+        return self.get(key)()
+
+    def help_topic(self, topic):
+        output = textwrap.dedent("""\
+            Bazaar directory formats
+            ------------------------
+
+            These formats can be used for creating branches, working trees, and
+            repositories.
+
+            """)
+        default_help = self.get_help('default')
+        help_pairs = []
+        for key in self.keys():
+            if key == 'default':
+                continue
+            help = self.get_help(key)
+            if help == default_help:
+                default_realkey = key
+            else:
+                help_pairs.append((key, help))
+
+        def wrapped(key, help, info):
+            if info.native:
+                help = '(native) ' + help
+            return '  %s:\n%s\n\n' % (key, 
+                    textwrap.fill(help, initial_indent='    ', 
+                    subsequent_indent='    '))
+        output += wrapped('%s/default' % default_realkey, default_help,
+                          self.get_info('default'))
+        deprecated_pairs = []
+        for key, help in help_pairs:
+            info = self.get_info(key)
+            if info.deprecated:
+                deprecated_pairs.append((key, help))
+            else:
+                output += wrapped(key, help, info)
+        if len(deprecated_pairs) > 0:
+            output += "Deprecated formats\n------------------\n\n"
+            for key, help in deprecated_pairs:
+                info = self.get_info(key)
+                output += wrapped(key, help, info)
+
+        return output
+
+
+format_registry = BzrDirFormatRegistry()
+format_registry.register('weave', BzrDirFormat6,
+    'Pre-0.8 format.  Slower than knit and does not'
+    ' support checkouts or shared repositories.', deprecated=True)
+format_registry.register_metadir('knit', 'RepositoryFormatKnit1',
+    'Format using knits.  Recommended.')
+format_registry.set_default('knit')
+format_registry.register_metadir('metaweave', 'RepositoryFormat7',
+    'Transitional format in 0.8.  Slower than knit.',
+    deprecated=True)
+format_registry.register_metadir('experimental-knit2', 'RepositoryFormatKnit2',
+    'Experimental successor to knit.  Use at your own risk.')

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-12-21 20:07:34 +0000
+++ bzrlib/errors.py	2007-01-04 20:59:46 +0000
@@ -541,6 +541,8 @@
 
     _fmt = "Lock error: %(message)s"
 
+    internal_error = True
+
     # All exceptions from the lock/unlock functions should be from
     # this exception class.  They will be translated as necessary. The
     # original exception is available as e.original_error
@@ -583,8 +585,6 @@
 
     _fmt = "%(obj)r is not locked"
 
-    internal_error = True
-
     # this can indicate that any particular object is not locked; see also
     # LockNotHeld which means that a particular *lock* object is not held by
     # the caller -- perhaps they should be unified.
@@ -611,7 +611,10 @@
 class LockContention(LockError):
 
     _fmt = "Could not acquire lock %(lock)s"
-    # TODO: show full url for lock, combining the transport and relative bits?
+    # TODO: show full url for lock, combining the transport and relative
+    # bits?
+
+    internal_error = False
     
     def __init__(self, lock):
         self.lock = lock
@@ -621,6 +624,8 @@
 
     _fmt = "Lock was broken while still open: %(lock)s - check storage consistency!"
 
+    internal_error = False
+
     def __init__(self, lock):
         self.lock = lock
 
@@ -629,6 +634,8 @@
 
     _fmt = "Lock was released and re-acquired before being broken: %(lock)s: held by %(holder)r, wanted to break %(target)r"
 
+    internal_error = False
+
     def __init__(self, lock, holder, target):
         self.lock = lock
         self.holder = holder
@@ -639,6 +646,8 @@
 
     _fmt = "Lock not held: %(lock)s"
 
+    internal_error = False
+
     def __init__(self, lock):
         self.lock = lock
 

=== modified file 'bzrlib/help_topics.py'
--- bzrlib/help_topics.py	2006-11-09 14:29:21 +0000
+++ bzrlib/help_topics.py	2007-01-10 06:01:49 +0000
@@ -105,7 +105,6 @@
 http://bazaar-vcs.org/
 
 Basic commands:
-
   bzr init           makes this directory a versioned branch
   bzr branch         make a copy of another branch
 
@@ -132,3 +131,7 @@
                         "Explain how to use --revision")
 topic_registry.register('basic', _basic_help, "Basic commands")
 topic_registry.register('topics', _help_on_topics, "Topics list")
+def get_format_topic(topic):
+    from bzrlib import bzrdir
+    return bzrdir.format_registry.help_topic(topic)
+topic_registry.register('formats', get_format_topic, 'Directory formats')

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-12-12 22:40:01 +0000
+++ bzrlib/inventory.py	2006-12-22 17:11:36 +0000
@@ -944,7 +944,7 @@
                 # if we finished all children, pop it off the stack
                 stack.pop()
 
-    def iter_entries_by_dir(self, from_dir=None):
+    def iter_entries_by_dir(self, from_dir=None, specific_file_ids=None):
         """Iterate over the entries in a directory first order.
 
         This returns all entries for a directory before returning
@@ -959,10 +959,34 @@
         if from_dir is None:
             if self.root is None:
                 return
+            # Optimize a common case
+            if specific_file_ids is not None and len(specific_file_ids) == 1:
+                file_id = list(specific_file_ids)[0]
+                if file_id in self:
+                    yield self.id2path(file_id), self[file_id]
+                return 
             from_dir = self.root
-            yield '', self.root
+            if (specific_file_ids is None or 
+                self.root.file_id in specific_file_ids):
+                yield '', self.root
         elif isinstance(from_dir, basestring):
             from_dir = self._byid[from_dir]
+
+        if specific_file_ids is not None:
+            parents = set()
+            def add_ancestors(file_id):
+                if file_id not in self:
+                    return
+                parent_id = self[file_id].parent_id
+                if parent_id is None:
+                    return
+                if parent_id not in parents:
+                    parents.add(parent_id)
+                    add_ancestors(parent_id)
+            for file_id in specific_file_ids:
+                add_ancestors(file_id)
+        else:
+            parents = None
             
         stack = [(u'', from_dir)]
         while stack:
@@ -973,10 +997,13 @@
 
                 child_relpath = cur_relpath + child_name
 
-                yield child_relpath, child_ie
+                if (specific_file_ids is None or 
+                    child_ie.file_id in specific_file_ids):
+                    yield child_relpath, child_ie
 
                 if child_ie.kind == 'directory':
-                    child_dirs.append((child_relpath+'/', child_ie))
+                    if parents is None or child_ie.file_id in parents:
+                        child_dirs.append((child_relpath+'/', child_ie))
             stack.extend(reversed(child_dirs))
 
     def entries(self):

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2006-12-20 18:31:23 +0000
+++ bzrlib/option.py	2007-01-12 03:52:12 +0000
@@ -26,6 +26,7 @@
 from bzrlib import (
     errors,
     revisionspec,
+    symbol_versioning,
     )
 """)
 from bzrlib.trace import warning
@@ -109,10 +110,11 @@
             (typestring, type_list)
         raise errors.BzrCommandError(msg)
 
+
 class Option(object):
     """Description of a command line option
     
-    :ivar short_name: If this option has a single-letter name, this is it.
+    :ivar _short_name: If this option has a single-letter name, this is it.
     Otherwise None.
     """
 
@@ -138,13 +140,27 @@
         self.name = name
         self.help = help
         self.type = type
-        self.short_name = short_name
+        self._short_name = short_name
         if type is None:
             assert argname is None
         elif argname is None:
             argname = 'ARG'
         self.argname = argname
 
+    def short_name(self):
+        if self._short_name:
+            return self._short_name
+        else:
+            # remove this when SHORT_OPTIONS is removed
+            # XXX: This is accessing a DeprecatedDict, so we call the super 
+            # method to avoid warnings
+            for (k, v) in dict.iteritems(Option.SHORT_OPTIONS):
+                if v == self:
+                    return k
+
+    def set_short_name(self, short_name):
+        self._short_name = short_name
+
     def get_negation_name(self):
         if self.name.startswith('no-'):
             return self.name[3:]
@@ -184,7 +200,7 @@
         argname =  self.argname
         if argname is not None:
             argname = argname.upper()
-        yield self.name, self.short_name, argname, self.help
+        yield self.name, self.short_name(), argname, self.help
 
 
 class OptionParser(optparse.OptionParser):
@@ -202,7 +218,7 @@
     parser = OptionParser()
     parser.remove_option('--help')
     for option in options.itervalues():
-        option.add_option(parser, option.short_name)
+        option.add_option(parser, option.short_name())
     return parser
 
 
@@ -262,3 +278,21 @@
 _global_option('dry-run',
                help="show what would be done, but don't actually do anything")
 _global_option('name-from-revision', help='The path name in the old tree.')
+
+
+# prior to 0.14 these were always globally registered; the old dict is
+# available for plugins that use it but it should not be used.
+Option.SHORT_OPTIONS = symbol_versioning.DeprecatedDict(
+    symbol_versioning.zero_fourteen,
+    'SHORT_OPTIONS',
+    {
+        'F': Option.OPTIONS['file'],
+        'h': Option.OPTIONS['help'],
+        'm': Option.OPTIONS['message'],
+        'r': Option.OPTIONS['revision'],
+        'v': Option.OPTIONS['verbose'],
+        'l': Option.OPTIONS['long'],
+        'q': Option.OPTIONS['quiet'],
+    },
+    'Set the short option name when constructing the Option.',
+    )

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-11-16 17:59:07 +0000
+++ bzrlib/osutils.py	2007-01-04 23:36:44 +0000
@@ -25,6 +25,8 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import codecs
+from datetime import datetime
 import errno
 from ntpath import (abspath as _nt_abspath,
                     join as _nt_join,
@@ -310,6 +312,8 @@
 rename = os.rename
 dirname = os.path.dirname
 basename = os.path.basename
+split = os.path.split
+splitext = os.path.splitext
 # These were already imported into local scope
 # mkdtemp = tempfile.mkdtemp
 # rmtree = shutil.rmtree
@@ -377,6 +381,17 @@
         output_encoding = bzrlib.user_encoding
         mutter('cp0 is invalid encoding.'
                ' encoding stdout as bzrlib.user_encoding %r', output_encoding)
+    # check encoding
+    try:
+        codecs.lookup(output_encoding)
+    except LookupError:
+        sys.stderr.write('bzr: warning:'
+                         ' unknown terminal encoding %s.\n'
+                         '  Using encoding %s instead.\n'
+                         % (output_encoding, bzrlib.user_encoding)
+                        )
+        output_encoding = bzrlib.user_encoding
+
     return output_encoding
 
 
@@ -552,14 +567,10 @@
 
 def local_time_offset(t=None):
     """Return offset of local zone from GMT, either at present or at time t."""
-    # python2.3 localtime() can't take None
     if t is None:
         t = time.time()
-        
-    if time.localtime(t).tm_isdst and time.daylight:
-        return -time.altzone
-    else:
-        return -time.timezone
+    offset = datetime.fromtimestamp(t) - datetime.utcfromtimestamp(t)
+    return offset.days * 86400 + offset.seconds
 
     
 def format_date(t, offset=0, timezone='original', date_fmt=None, 
@@ -1057,17 +1068,21 @@
 _cached_user_encoding = None
 
 
-def get_user_encoding():
+def get_user_encoding(use_cache=True):
     """Find out what the preferred user encoding is.
 
     This is generally the encoding that is used for command line parameters
     and file contents. This may be different from the terminal encoding
     or the filesystem encoding.
 
+    :param  use_cache:  Enable cache for detected encoding.
+                        (This parameter is turned on by default,
+                        and required only for selftesting)
+
     :return: A string defining the preferred user encoding
     """
     global _cached_user_encoding
-    if _cached_user_encoding is not None:
+    if _cached_user_encoding is not None and use_cache:
         return _cached_user_encoding
 
     if sys.platform == 'darwin':
@@ -1081,7 +1096,7 @@
         import locale
 
     try:
-        _cached_user_encoding = locale.getpreferredencoding()
+        user_encoding = locale.getpreferredencoding()
     except locale.Error, e:
         sys.stderr.write('bzr: warning: %s\n'
                          '  Could not determine what text encoding to use.\n'
@@ -1089,13 +1104,29 @@
                          '  doesn\'t support the locale set by $LANG (%s)\n'
                          "  Continuing with ascii encoding.\n"
                          % (e, os.environ.get('LANG')))
+        user_encoding = 'ascii'
 
     # Windows returns 'cp0' to indicate there is no code page. So we'll just
     # treat that as ASCII, and not support printing unicode characters to the
     # console.
-    if _cached_user_encoding in (None, 'cp0'):
-        _cached_user_encoding = 'ascii'
-    return _cached_user_encoding
+    if user_encoding in (None, 'cp0'):
+        user_encoding = 'ascii'
+    else:
+        # check encoding
+        try:
+            codecs.lookup(user_encoding)
+        except LookupError:
+            sys.stderr.write('bzr: warning:'
+                             ' unknown encoding %s.'
+                             ' Continuing with ascii encoding.\n'
+                             % user_encoding
+                            )
+            user_encoding = 'ascii'
+
+    if use_cache:
+        _cached_user_encoding = user_encoding
+
+    return user_encoding
 
 
 def recv_all(socket, bytes):

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2006-11-10 05:57:17 +0000
+++ bzrlib/plugin.py	2007-01-03 00:23:45 +0000
@@ -113,6 +113,7 @@
     dirs.insert(0, os.path.dirname(plugins.__file__))
 
     load_from_dirs(dirs)
+    load_from_zips(dirs)
 
 
 def load_from_dirs(dirs):
@@ -183,3 +184,86 @@
                 ## import pdb; pdb.set_trace()
                 warning('Unable to load plugin %r from %r' % (name, d))
                 log_exception_quietly()
+
+
+def load_from_zips(zips):
+    """Load bzr plugins from zip archives with zipimport.
+    It's similar to load_from_dirs but plugins searched inside archives.
+    """
+    import zipfile
+    import zipimport
+
+    valid_suffixes = ('.py', '.pyc', '.pyo')    # only python modules/packages
+                                                # is allowed
+    for zip_name in zips:
+        if '.zip' not in zip_name:
+            continue
+        try:
+            ziobj = zipimport.zipimporter(zip_name)
+        except zipimport.ZipImportError:
+            # not a valid zip
+            continue
+        mutter('Looking for plugins in %r', zip_name)
+
+        # use zipfile to get list of files/dirs inside zip
+        z = zipfile.ZipFile(ziobj.archive)
+        namelist = z.namelist()
+        z.close()
+
+        if ziobj.prefix:
+            prefix = ziobj.prefix.replace('\\','/')
+            ix = len(prefix)
+            namelist = [name[ix:]
+                        for name in namelist
+                        if name.startswith(prefix)]
+
+        mutter('Names in archive: %r', namelist)
+
+        for name in namelist:
+            if not name or name.endswith('/'):
+                continue
+
+            # '/' is used to separate pathname components inside zip archives
+            ix = name.rfind('/')
+            if ix == -1:
+                head, tail = '', name
+            else:
+                head, tail = name.rsplit('/',1)
+            if '/' in head:
+                # we don't need looking in subdirectories
+                continue
+
+            base, suffix = osutils.splitext(tail)
+            if suffix not in valid_suffixes:
+                continue
+
+            if base == '__init__':
+                # package
+                plugin_name = head
+            elif head == '':
+                # module
+                plugin_name = base
+            else:
+                continue
+
+            if not plugin_name:
+                continue
+            if getattr(plugins, plugin_name, None):
+                mutter('Plugin name %s already loaded', plugin_name)
+                continue
+
+            try:
+                plugin = ziobj.load_module(plugin_name)
+                setattr(plugins, plugin_name, plugin)
+                mutter('Load plugin %s from zip %r', plugin_name, zip_name)
+            except zipimport.ZipImportError, e:
+                mutter('Unable to load plugin %r from %r: %s',
+                       plugin_name, zip_name, str(e))
+                continue
+            except KeyboardInterrupt:
+                raise
+            except Exception, e:
+                ## import pdb; pdb.set_trace()
+                warning('Unable to load plugin %r from %r'
+                        % (name, zip_name))
+                log_exception_quietly()

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-12-18 19:21:11 +0000
+++ bzrlib/repository.py	2007-01-05 06:32:57 +0000
@@ -1278,7 +1278,12 @@
         klass._formats[format.get_format_string()] = format
 
     @classmethod
+    @deprecated_method(symbol_versioning.zero_fourteen)
     def set_default_format(klass, format):
+        klass._set_default_format(format)
+
+    @classmethod
+    def _set_default_format(klass, format):
         klass._default_format = format
 
     @classmethod
@@ -1800,10 +1805,11 @@
 # formats which have no format string are not discoverable
 # and not independently creatable, so are not registered.
 RepositoryFormat.register_format(RepositoryFormat7())
+# KEEP in sync with bzrdir.format_registry default
 _default_format = RepositoryFormatKnit1()
 RepositoryFormat.register_format(_default_format)
 RepositoryFormat.register_format(RepositoryFormatKnit2())
-RepositoryFormat.set_default_format(_default_format)
+RepositoryFormat._set_default_format(_default_format)
 _legacy_formats = [RepositoryFormat4(),
                    RepositoryFormat5(),
                    RepositoryFormat6()]

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2006-11-27 23:11:54 +0000
+++ bzrlib/symbol_versioning.py	2007-01-16 15:56:53 +0000
@@ -1,5 +1,5 @@
-# Copyright (C) 2006 Canonical Ltd
-#   Authors: Robert Collins <robert.collins@canonical.com>
+# Copyright (C) 2006, 2007 Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com> and others
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
            'zero_twelve',
            'zero_thirteen',
            'zero_fourteen',
+           'zero_fifteen',
            ]
 
 from warnings import warn
@@ -47,6 +48,7 @@
 zero_twelve = "%s was deprecated in version 0.12."
 zero_thirteen = "%s was deprecated in version 0.13."
 zero_fourteen = "%s was deprecated in version 0.14."
+zero_fifteen = "%s was deprecated in version 0.15."
 
 
 def set_warning_method(method):
@@ -176,6 +178,52 @@
     decorated_callable.is_deprecated = True
 
 
+def _dict_deprecation_wrapper(wrapped_method):
+    """Returns a closure that emits a warning and calls the superclass"""
+    def cb(dep_dict, *args, **kwargs):
+        msg = 'access to %s' % (dep_dict._variable_name, )
+        msg = dep_dict._deprecation_version % (msg,)
+        if dep_dict._advice:
+            msg += ' ' + dep_dict._advice
+        warn(msg, DeprecationWarning, stacklevel=2)
+        return wrapped_method(dep_dict, *args, **kwargs)
+    return cb
+
+
+class DeprecatedDict(dict):
+    """A dictionary that complains when read or written."""
+
+    is_deprecated = True
+
+    def __init__(self,
+        deprecation_version,
+        variable_name,
+        initial_value,
+        advice,
+        ):
+        """Create a dict that warns when read or modified.
+
+        :param deprecation_version: something like zero_nine
+        :param initial_value: The contents of the dict
+        :param variable_name: This allows better warnings to be printed
+        :param advice: String of advice on what callers should do instead 
+            of using this variable.
+        """
+        self._deprecation_version = deprecation_version
+        self._variable_name = variable_name
+        self._advice = advice
+        dict.__init__(self, initial_value)
+
+    # This isn't every possible method but it should trap anyone using the
+    # dict -- add more if desired
+    __len__ = _dict_deprecation_wrapper(dict.__len__)
+    __getitem__ = _dict_deprecation_wrapper(dict.__getitem__)
+    __setitem__ = _dict_deprecation_wrapper(dict.__setitem__)
+    __delitem__ = _dict_deprecation_wrapper(dict.__delitem__)
+    keys = _dict_deprecation_wrapper(dict.keys)
+    __contains__ = _dict_deprecation_wrapper(dict.__contains__)
+
+
 def deprecated_list(deprecation_version, variable_name,
                     initial_value, extra=None):
     """Create a list that warns when modified

=== modified file 'bzrlib/tests/HTTPTestUtil.py'
--- bzrlib/tests/HTTPTestUtil.py	2006-12-15 02:47:54 +0000
+++ bzrlib/tests/HTTPTestUtil.py	2006-12-22 09:54:52 +0000
@@ -18,6 +18,7 @@
 import errno
 from SimpleHTTPServer import SimpleHTTPRequestHandler
 import socket
+import urlparse
 
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests.HttpServer import (
@@ -206,7 +207,22 @@
     """Append a '-proxied' suffix to file served"""
 
     def translate_path(self, path):
-        return TestingHTTPRequestHandler.translate_path(self,
-                                                        path + '-proxied')
+        # We need to act as a proxy and accept absolute urls,
+        # which SimpleHTTPRequestHandler (grand parent) is not
+        # ready for. So we just drop the protocol://host:port
+        # part in front of the request-url (because we know we
+        # would not forward the request to *another* proxy).
+
+        # So we do what SimpleHTTPRequestHandler.translate_path
+        # do beginning with python 2.4.3: abandon query
+        # parameters, scheme, host port, etc (which ensure we
+        # provide the right behaviour on all python versions).
+        path = urlparse.urlparse(path)[2]
+        # And now, we can apply *our* trick to proxy files
+        self.path += '-proxied'
+        # An finally we leave our mother class do whatever it
+        # wants with the path
+        return TestingHTTPRequestHandler.translate_path(self, path)
+
 
 

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-12-19 07:40:19 +0000
+++ bzrlib/tests/__init__.py	2007-01-17 01:47:02 +0000
@@ -1262,14 +1262,6 @@
         self.transport_server = default_transport
         self.transport_readonly_server = None
 
-    def failUnlessExists(self, path):
-        """Fail unless path, which may be abs or relative, exists."""
-        self.failUnless(osutils.lexists(path))
-
-    def failIfExists(self, path):
-        """Fail if path, which may be abs or relative, exists."""
-        self.failIf(osutils.lexists(path))
-        
     def get_transport(self):
         """Return a writeable transport for the test scratch space"""
         t = get_transport(self.get_url())
@@ -1413,7 +1405,9 @@
                     t.mkdir('.')
                 except errors.FileExists:
                     pass
-            if format is None:
+            if isinstance(format, basestring):
+                format = bzrdir.format_registry.make_bzrdir(format)
+            elif format is None:
                 format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
             return format.initialize_on_transport(t)
         except errors.UninitializableFormat:
@@ -1529,16 +1523,9 @@
                 elif line_endings == 'native':
                     end = os.linesep
                 else:
-                    raise errors.BzrError('Invalid line ending request %r' % (line_endings,))
+                    raise errors.BzrError(
+                        'Invalid line ending request %r' % line_endings)
                 content = "contents of %s%s" % (name.encode('utf-8'), end)
-                # Technically 'put()' is the right command. However, put
-                # uses an AtomicFile, which requires an extra rename into place
-                # As long as the files didn't exist in the past, append() will
-                # do the same thing as put()
-                # On jam's machine, make_kernel_like_tree is:
-                #   put:    4.5-7.5s (averaging 6s)
-                #   append: 2.9-4.5s
-                #   put_non_atomic: 2.9-4.5s
                 transport.put_bytes_non_atomic(urlutils.escape(name), content)
 
     def build_tree_contents(self, shape):
@@ -1546,10 +1533,18 @@
 
     def assertFileEqual(self, content, path):
         """Fail if path does not contain 'content'."""
-        self.failUnless(osutils.lexists(path))
+        self.failUnlessExists(path)
         # TODO: jam 20060427 Shouldn't this be 'rb'?
         self.assertEqualDiff(content, open(path, 'r').read())
 
+    def failUnlessExists(self, path):
+        """Fail unless path, which may be abs or relative, exists."""
+        self.failUnless(osutils.lexists(path))
+
+    def failIfExists(self, path):
+        """Fail if path, which may be abs or relative, exists."""
+        self.failIf(osutils.lexists(path))
+
 
 class TestCaseWithTransport(TestCaseInTempDir):
     """A test case that provides get_url and get_readonly_url facilities.
@@ -1663,9 +1658,22 @@
     return result
 
 
+def sort_suite_by_re(suite, pattern):
+    first = []
+    second = []
+    filter_re = re.compile(pattern)
+    for test in iter_suite_tests(suite):
+        if filter_re.search(test.id()):
+            first.append(test)
+        else:
+            second.append(test)
+    return TestUtil.TestSuite(first + second)
+
+
 def run_suite(suite, name='test', verbose=False, pattern=".*",
               stop_on_failure=False, keep_output=False,
-              transport=None, lsprof_timed=None, bench_history=None):
+              transport=None, lsprof_timed=None, bench_history=None,
+              matching_tests_first=None):
     TestCase._gather_lsprof_in_benchmarks = lsprof_timed
     if verbose:
         verbosity = 2
@@ -1678,7 +1686,10 @@
                             bench_history=bench_history)
     runner.stop_on_failure=stop_on_failure
     if pattern != '.*':
-        suite = filter_suite_by_re(suite, pattern)
+        if matching_tests_first:
+            suite = sort_suite_by_re(suite, pattern)
+        else:
+            suite = filter_suite_by_re(suite, pattern)
     result = runner.run(suite)
     return result.wasSuccessful()
 
@@ -1688,7 +1699,8 @@
              transport=None,
              test_suite_factory=None,
              lsprof_timed=None,
-             bench_history=None):
+             bench_history=None,
+             matching_tests_first=None):
     """Run the whole test suite under the enhanced runner"""
     # XXX: Very ugly way to do this...
     # Disable warning about old formats because we don't want it to disturb
@@ -1710,7 +1722,8 @@
                      stop_on_failure=stop_on_failure, keep_output=keep_output,
                      transport=transport,
                      lsprof_timed=lsprof_timed,
-                     bench_history=bench_history)
+                     bench_history=bench_history,
+                     matching_tests_first=matching_tests_first)
     finally:
         default_transport = old_transport
 
@@ -1743,6 +1756,7 @@
                    'bzrlib.tests.test_escaped_store',
                    'bzrlib.tests.test_fetch',
                    'bzrlib.tests.test_ftp_transport',
+                   'bzrlib.tests.test_generate_docs',
                    'bzrlib.tests.test_generate_ids',
                    'bzrlib.tests.test_globbing',
                    'bzrlib.tests.test_gpg',
@@ -1768,6 +1782,7 @@
                    'bzrlib.tests.test_nonascii',
                    'bzrlib.tests.test_options',
                    'bzrlib.tests.test_osutils',
+                   'bzrlib.tests.test_osutils_encodings',
                    'bzrlib.tests.test_patch',
                    'bzrlib.tests.test_patches',
                    'bzrlib.tests.test_permissions',

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2007-01-05 17:12:03 +0000
@@ -20,6 +20,9 @@
 import os
 from cStringIO import StringIO
 
+from bzrlib import (
+    bzrdir,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import (BzrDir, BzrDirFormat, BzrDirMetaFormat1)
 from bzrlib.osutils import getcwd
@@ -100,13 +103,8 @@
         self.run_bzr('unbind', retcode=3)
 
     def init_meta_branch(self, path):
-        old_format = BzrDirFormat.get_default_format()
-        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
-        try:
-            return BzrDir.create_branch_convenience(
-                path, BzrDirMetaFormat1())
-        finally:
-            BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        return BzrDir.create_branch_convenience(path, format=format)
 
     def test_bound_commit(self):
         bzr = self.run_bzr

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2006-10-16 01:50:48 +0000
+++ bzrlib/tests/blackbox/test_info.py	2007-01-05 17:12:03 +0000
@@ -21,7 +21,10 @@
 import sys
 
 import bzrlib
-from bzrlib import repository
+from bzrlib import (
+    bzrdir,
+    repository,
+    )
 from bzrlib.osutils import format_date
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
@@ -42,10 +45,7 @@
         transport = self.get_transport()
 
         # Create initial standalone branch
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirFormat6())
-        tree1 = self.make_branch_and_tree('standalone')
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        tree1 = self.make_branch_and_tree('standalone', 'weave')
         self.build_tree(['standalone/a'])
         tree1.add('a')
         branch1 = tree1.branch
@@ -134,8 +134,7 @@
         # Branch and bind to standalone, needs upgrade to metadir
         # (creates backup as unknown)
         branch1.bzrdir.sprout('bound')
-        knit1_format = bzrlib.bzrdir.BzrDirMetaFormat1()
-        knit1_format.repository_format = repository.RepositoryFormatKnit1()
+        knit1_format = bzrdir.format_registry.make_bzrdir('knit')
         bzrlib.upgrade.upgrade('bound', knit1_format)
         branch3 = bzrlib.bzrdir.BzrDir.open('bound').open_branch()
         branch3.bind(branch1)
@@ -188,10 +187,8 @@
         self.assertEqual('', err)
 
         # Checkout standalone (same as above, but does not have parent set)
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(knit1_format)
-        branch4 = bzrlib.bzrdir.BzrDir.create_branch_convenience('checkout')
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        branch4 = bzrlib.bzrdir.BzrDir.create_branch_convenience('checkout',
+            format=knit1_format)
         branch4.bind(branch1)
         branch4.bzrdir.open_workingtree().update()
         out, err = self.runbzr('info checkout --verbose')
@@ -238,14 +235,7 @@
         self.assertEqual('', err)
 
         # Lightweight checkout (same as above, different branch and repository)
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
-        transport.mkdir('lightcheckout')
-        dir5 = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('lightcheckout')
-        bzrlib.branch.BranchReferenceFormat().initialize(dir5, branch1)
-        dir5.create_workingtree()
-        tree5 = dir5.open_workingtree()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        tree5 = branch1.create_checkout('lightcheckout', lightweight=True)
         branch5 = tree5.branch
         out, err = self.runbzr('info lightcheckout')
         self.assertEqualDiff(
@@ -470,6 +460,7 @@
 
     def test_info_standalone_no_tree(self):
         # create standalone branch without a working tree
+        format = bzrdir.format_registry.make_bzrdir('default')
         branch = self.make_branch('branch')
         repo = branch.repository
         out, err = self.runbzr('info branch')
@@ -489,17 +480,16 @@
          0 revisions
          0 KiB
 """ % (branch.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
     def test_info_shared_repository(self):
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        format = bzrdir.format_registry.make_bzrdir('metaweave')
         transport = self.get_transport()
 
         # Create shared repository
-        repo = self.make_repository('repo', shared=True)
+        repo = self.make_repository('repo', shared=True, format=format)
         repo.set_make_working_trees(False)
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
@@ -514,13 +504,14 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
         # Create branch inside shared repository
         repo.bzrdir.root_transport.mkdir('branch')
-        branch1 = repo.bzrdir.create_branch_convenience('repo/branch')
+        branch1 = repo.bzrdir.create_branch_convenience('repo/branch',
+            format=format)
         out, err = self.runbzr('info repo/branch')
         self.assertEqualDiff(
 """Location:
@@ -539,19 +530,18 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
         # Create lightweight checkout
         transport.mkdir('tree')
         transport.mkdir('tree/lightcheckout')
-        dir2 = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('tree/lightcheckout')
-        bzrlib.branch.BranchReferenceFormat().initialize(dir2, branch1)
-        dir2.create_workingtree()
-        tree2 = dir2.open_workingtree()
+        tree2 = branch1.create_checkout('tree/lightcheckout', 
+            lightweight=True)
         branch2 = tree2.branch
-        self.assertCheckoutStatusOutput('tree/lightcheckout', tree2, shared_repo=repo)
+        self.assertCheckoutStatusOutput('tree/lightcheckout', tree2, 
+                   shared_repo=repo)
 
         # Create normal checkout
         tree3 = branch1.create_checkout('tree/checkout')
@@ -599,7 +589,7 @@
          %d KiB
 """ % (tree2.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -640,7 +630,7 @@
          0 KiB
 """ % (tree3.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
@@ -681,7 +671,7 @@
          1 revision
          %d KiB
 """ % (tree3.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -730,7 +720,7 @@
          %d KiB
 """ % (tree2.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_last,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -761,7 +751,7 @@
          2 revisions
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_last,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -783,22 +773,19 @@
          2 revisions
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
        repo._revision_store.total_size(repo.get_transaction())[1] / 1024,
        ), out)
         self.assertEqual('', err)
 
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
-
     def test_info_shared_repository_with_trees(self):
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        format = bzrdir.format_registry.make_bzrdir('metaweave')
         transport = self.get_transport()
 
         # Create shared repository with working trees
-        repo = self.make_repository('repo', shared=True)
+        repo = self.make_repository('repo', shared=True, format=format)
         repo.set_make_working_trees(True)
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
@@ -815,13 +802,14 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
         # Create two branches
         repo.bzrdir.root_transport.mkdir('branch1')
-        branch1 = repo.bzrdir.create_branch_convenience('repo/branch1')
+        branch1 = repo.bzrdir.create_branch_convenience('repo/branch1',
+            format=format)
         branch2 = branch1.bzrdir.sprout('repo/branch2').open_branch()
 
         # Empty first branch
@@ -855,7 +843,7 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
@@ -898,7 +886,7 @@
          1 revision
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -941,7 +929,7 @@
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
        repo._revision_store.total_size(repo.get_transaction())[1] / 1024,
@@ -987,7 +975,7 @@
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -1011,23 +999,20 @@
          1 revision
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
        repo._revision_store.total_size(repo.get_transaction())[1] / 1024,
        ),
        out)
         self.assertEqual('', err)
-
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
     
     def test_info_shared_repository_with_tree_in_root(self):
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        format = bzrdir.format_registry.make_bzrdir('metaweave')
         transport = self.get_transport()
 
         # Create shared repository with working trees
-        repo = self.make_repository('repo', shared=True)
+        repo = self.make_repository('repo', shared=True, format=format)
         repo.set_make_working_trees(True)
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
@@ -1044,7 +1029,7 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
@@ -1081,12 +1066,10 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
-
     def assertCheckoutStatusOutput(self, 
         command_string, lco_tree, shared_repo=None,
         repo_branch=None,

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2006-11-02 10:20:19 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2007-01-05 17:12:03 +0000
@@ -35,35 +35,23 @@
     def setUp(self):
         super(TestWithUpgradableBranches, self).setUp()
         self.old_format = bzrdir.BzrDirFormat.get_default_format()
-        self.old_repo_format = \
-            bzrlib.repository.RepositoryFormat.get_default_format()
         self.old_ui_factory = ui.ui_factory
         self.addCleanup(self.restoreDefaults)
 
         ui.ui_factory = TestUIFactory()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        bzrlib.repository.RepositoryFormat.set_default_format(
-            bzrlib.repository.RepositoryFormat7())
-        # FIXME RBC 20060120 we should be able to do this via ui calls only.
         # setup a format 5 branch we can upgrade from.
-        t = get_transport(self.get_url())
-        t.mkdir('format_5_branch')
-        bzrdir.BzrDirFormat5().initialize(self.get_url('format_5_branch'))
-        bzrdir.BzrDir.create_standalone_workingtree('current_format_branch')
-        d = bzrdir.BzrDir.create('metadir_weave_branch')
-        d.create_repository()
-        d.create_branch()
-        d.create_workingtree()
-        self.run_bzr('checkout',
-                     '--lightweight',
-                     self.get_url('current_format_branch'),
-                     'current_format_checkout')
+        self.make_branch_and_tree('format_5_branch',
+                                  format=bzrdir.BzrDirFormat5())
+
+        current_tree = self.make_branch_and_tree('current_format_branch',
+                                                 format='default')
+        self.make_branch_and_tree('metadir_weave_branch', format='metaweave')
+        current_tree.branch.create_checkout(
+            self.get_url('current_format_checkout'), lightweight=True)
 
     def restoreDefaults(self):
-        bzrdir.BzrDirFormat.set_default_format(self.old_format)
-        bzrlib.repository.RepositoryFormat.set_default_format(
-            self.old_repo_format)
         ui.ui_factory = self.old_ui_factory
+        bzrdir.BzrDirFormat._set_default_format(self.old_format)
 
     def test_readonly_url_error(self):
         (out, err) = self.run_bzr_captured(
@@ -111,7 +99,7 @@
         # users can force an upgrade to metadir format.
         url = get_transport(self.get_url('format_5_branch')).base
         # check --format takes effect
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirFormat5())
+        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         (out, err) = self.run_bzr_captured(
             ['upgrade', '--format=metaweave', url])
         self.assertEqualDiff("""starting upgrade of %s
@@ -135,7 +123,7 @@
         # branch
         url = get_transport(self.get_url('metadir_weave_branch')).base
         # check --format takes effect
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirFormat5())
+        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         (out, err) = self.run_bzr_captured(
             ['upgrade', '--format=knit', url])
         self.assertEqualDiff("""starting upgrade of %s

=== modified file 'bzrlib/tests/branch_implementations/test_bound_sftp.py'
--- bzrlib/tests/branch_implementations/test_bound_sftp.py	2006-09-11 01:03:59 +0000
+++ bzrlib/tests/branch_implementations/test_bound_sftp.py	2007-01-05 17:12:03 +0000
@@ -20,6 +20,9 @@
 import os
 
 import bzrlib
+from bzrlib import (
+    bzrdir,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import (BzrDir,
                            BzrDirFormat,
@@ -34,12 +37,9 @@
 
     def create_branches(self):
         self.build_tree(['base/', 'base/a', 'base/b'])
-        old_format = BzrDirFormat.get_default_format()
-        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
-        try:
-            wt_base = BzrDir.create_standalone_workingtree('base')
-        finally:
-            BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        wt_base = BzrDir.create_standalone_workingtree('base',
+            format=format)
     
         b_base = wt_base.branch
 
@@ -72,12 +72,8 @@
         # manually make a branch we can bind, because the default format
         # may not be bindable-from, and we want to test the side effects etc
         # of bondage.
-        old_format = BzrDirFormat.get_default_format()
-        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
-        try:
-            b_child = BzrDir.create_branch_convenience('child')
-        finally:
-            BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        b_child = BzrDir.create_branch_convenience('child', format=format)
         self.assertEqual(None, b_child.get_bound_location())
         self.assertEqual(None, b_child.get_master_branch())
 

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-10-02 13:05:18 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-01-09 06:57:43 +0000
@@ -1261,15 +1261,10 @@
         dir = self.make_bzrdir('.')
         if dir.can_convert_format():
             # if its default updatable there must be an updater 
-            # (we change the default to match the lastest known format
-            # as downgrades may not be available
-            old_format = bzrdir.BzrDirFormat.get_default_format()
-            bzrdir.BzrDirFormat.set_default_format(dir._format)
-            try:
-                self.assertTrue(isinstance(dir._format.get_converter(),
-                                           bzrdir.Converter))
-            finally:
-                bzrdir.BzrDirFormat.set_default_format(old_format)
+            # (we force the latest known format as downgrades may not be
+            # available
+            self.assertTrue(isinstance(dir._format.get_converter(
+                format=dir._format), bzrdir.Converter))
         dir.needs_format_conversion(None)
 
     def test_upgrade_new_instance(self):
@@ -1281,15 +1276,12 @@
         self.createWorkingTreeOrSkip(dir)
         if dir.can_convert_format():
             # if its default updatable there must be an updater 
-            # (we change the default to match the lastest known format
-            # as downgrades may not be available
-            old_format = bzrdir.BzrDirFormat.get_default_format()
-            bzrdir.BzrDirFormat.set_default_format(dir._format)
+            # (we force the latest known format as downgrades may not be
+            # available
             pb = ui.ui_factory.nested_progress_bar()
             try:
-                dir._format.get_converter(None).convert(dir, pb)
+                dir._format.get_converter(format=dir._format).convert(dir, pb)
             finally:
-                bzrdir.BzrDirFormat.set_default_format(old_format)
                 pb.finished()
             # and it should pass 'check' now.
             check(bzrdir.BzrDir.open(self.get_url('.')).open_branch(), False)

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2006-10-02 13:05:18 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2007-01-05 17:12:03 +0000
@@ -273,11 +273,8 @@
             old_format = bzrdir.BzrDirFormat.get_default_format()
             # This gives metadir branches something they can convert to.
             # it would be nice to have a 'latest' vs 'default' concept.
-            bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-            try:
-                upgrade(wt.basedir)
-            finally:
-                bzrdir.BzrDirFormat.set_default_format(old_format)
+            format = bzrdir.format_registry.make_bzrdir('experimental-knit2')
+            upgrade(wt.basedir, format=format)
         except errors.UpToDateFormat:
             # this is in the most current format already.
             return

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2007-01-11 14:26:18 +0000
+++ bzrlib/tests/test_bzrdir.py	2007-01-17 14:33:15 +0000
@@ -23,6 +23,8 @@
 from StringIO import StringIO
 
 from bzrlib import (
+    help_topics,
+    symbol_versioning,
     urlutils,
     )
 import bzrlib.branch
@@ -46,16 +48,92 @@
         old_format = bzrdir.BzrDirFormat.get_default_format()
         # default is BzrDirFormat6
         self.failUnless(isinstance(old_format, bzrdir.BzrDirMetaFormat1))
-        bzrdir.BzrDirFormat.set_default_format(SampleBzrDirFormat())
+        self.applyDeprecated(symbol_versioning.zero_fourteen, 
+                             bzrdir.BzrDirFormat.set_default_format, 
+                             SampleBzrDirFormat())
         # creating a bzr dir should now create an instrumented dir.
         try:
             result = bzrdir.BzrDir.create('memory:///')
             self.failUnless(isinstance(result, SampleBzrDir))
         finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+            self.applyDeprecated(symbol_versioning.zero_fourteen,
+                bzrdir.BzrDirFormat.set_default_format, old_format)
         self.assertEqual(old_format, bzrdir.BzrDirFormat.get_default_format())
 
 
+class TestFormatRegistry(TestCase):
+
+    def make_format_registry(self):
+        my_format_registry = bzrdir.BzrDirFormatRegistry()
+        my_format_registry.register('weave', bzrdir.BzrDirFormat6,
+            'Pre-0.8 format.  Slower and does not support checkouts or shared'
+            ' repositories', deprecated=True)
+        my_format_registry.register_lazy('lazy', 'bzrlib.bzrdir', 
+            'BzrDirFormat6', 'Format registered lazily', deprecated=True)
+        my_format_registry.register_metadir('knit', 'RepositoryFormatKnit1',
+            'Format using knits')
+        my_format_registry.set_default('knit')
+        my_format_registry.register_metadir('metaweave', 'RepositoryFormat7',
+            'Transitional format in 0.8.  Slower than knit.', deprecated=True)
+        my_format_registry.register_metadir('experimental-knit2', 
+                                            'RepositoryFormatKnit2',
+            'Experimental successor to knit.  Use at your own risk.')
+        return my_format_registry
+
+    def test_format_registry(self):
+        my_format_registry = self.make_format_registry()
+        my_bzrdir = my_format_registry.make_bzrdir('lazy')
+        self.assertIsInstance(my_bzrdir, bzrdir.BzrDirFormat6)
+        my_bzrdir = my_format_registry.make_bzrdir('weave')
+        self.assertIsInstance(my_bzrdir, bzrdir.BzrDirFormat6)
+        my_bzrdir = my_format_registry.make_bzrdir('default')
+        self.assertIsInstance(my_bzrdir.repository_format, 
+            repository.RepositoryFormatKnit1)
+        my_bzrdir = my_format_registry.make_bzrdir('knit')
+        self.assertIsInstance(my_bzrdir.repository_format, 
+            repository.RepositoryFormatKnit1)
+        my_bzrdir = my_format_registry.make_bzrdir('metaweave')
+        self.assertIsInstance(my_bzrdir.repository_format, 
+            repository.RepositoryFormat7)
+
+    def test_get_help(self):
+        my_format_registry = self.make_format_registry()
+        self.assertEqual('Format registered lazily',
+                         my_format_registry.get_help('lazy'))
+        self.assertEqual('Format using knits', 
+                         my_format_registry.get_help('knit'))
+        self.assertEqual('Format using knits', 
+                         my_format_registry.get_help('default'))
+        self.assertEqual('Pre-0.8 format.  Slower and does not support'
+                         ' checkouts or shared repositories', 
+                         my_format_registry.get_help('weave'))
+        
+    def test_help_topic(self):
+        topics = help_topics.HelpTopicRegistry()
+        topics.register('formats', self.make_format_registry().help_topic, 
+                        'Directory formats')
+        topic = topics.get_detail('formats')
+        new, deprecated = topic.split('Deprecated formats')
+        self.assertContainsRe(new, 'Bazaar directory formats')
+        self.assertContainsRe(new, 
+            '  knit/default:\n    \(native\) Format using knits\n')
+        self.assertContainsRe(deprecated, 
+            '  lazy:\n    \(native\) Format registered lazily\n')
+
+    def test_set_default_repository(self):
+        default_factory = bzrdir.format_registry.get('default')
+        old_default = [k for k, v in bzrdir.format_registry.iteritems()
+                       if v == default_factory and k != 'default'][0]
+        bzrdir.format_registry.set_default_repository('metaweave')
+        try:
+            self.assertIs(bzrdir.format_registry.get('metaweave'),
+                          bzrdir.format_registry.get('default'))
+            self.assertIs(
+                repository.RepositoryFormat.get_default_format().__class__,
+                repository.RepositoryFormat7)
+        finally:
+            bzrdir.format_registry.set_default_repository(old_default)
+
 class SampleBranch(bzrlib.branch.Branch):
     """A dummy branch for guess what, dummy use."""
 
@@ -159,13 +237,8 @@
 
     def test_create_repository(self):
         format = SampleBzrDirFormat()
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(format)
-        try:
-            repo = bzrdir.BzrDir.create_repository(self.get_url())
-            self.assertEqual('A repository', repo)
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        repo = bzrdir.BzrDir.create_repository(self.get_url(), format=format)
+        self.assertEqual('A repository', repo)
 
     def test_create_repository_shared(self):
         old_format = bzrdir.BzrDirFormat.get_default_format()
@@ -180,183 +253,138 @@
     def test_create_repository_under_shared(self):
         # an explicit create_repository always does so.
         # we trust the format is right from the 'create_repository test'
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            self.make_repository('.', shared=True)
-            repo = bzrdir.BzrDir.create_repository(self.get_url('child'))
-            self.assertTrue(isinstance(repo, repository.Repository))
-            self.assertTrue(repo.bzrdir.root_transport.base.endswith('child/'))
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        self.make_repository('.', shared=True, format=format)
+        repo = bzrdir.BzrDir.create_repository(self.get_url('child'),
+                                               format=format)
+        self.assertTrue(isinstance(repo, repository.Repository))
+        self.assertTrue(repo.bzrdir.root_transport.base.endswith('child/'))
 
     def test_create_branch_and_repo_uses_default(self):
         format = SampleBzrDirFormat()
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(format)
-        try:
-            branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url())
-            self.assertTrue(isinstance(branch, SampleBranch))
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url(), 
+                                                      format=format)
+        self.assertTrue(isinstance(branch, SampleBranch))
 
     def test_create_branch_and_repo_under_shared(self):
         # creating a branch and repo in a shared repo uses the
         # shared repository
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            self.make_repository('.', shared=True)
-            branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url('child'))
-            self.assertRaises(errors.NoRepositoryPresent,
-                              branch.bzrdir.open_repository)
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        self.make_repository('.', shared=True, format=format)
+        branch = bzrdir.BzrDir.create_branch_and_repo(
+            self.get_url('child'), format=format)
+        self.assertRaises(errors.NoRepositoryPresent,
+                          branch.bzrdir.open_repository)
 
     def test_create_branch_and_repo_under_shared_force_new(self):
         # creating a branch and repo in a shared repo can be forced to 
         # make a new repo
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            self.make_repository('.', shared=True)
-            branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url('child'),
-                                                          force_new_repo=True)
-            branch.bzrdir.open_repository()
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        self.make_repository('.', shared=True, format=format)
+        branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url('child'),
+                                                      force_new_repo=True,
+                                                      format=format)
+        branch.bzrdir.open_repository()
 
     def test_create_standalone_working_tree(self):
         format = SampleBzrDirFormat()
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(format)
-        try:
-            # note this is deliberately readonly, as this failure should 
-            # occur before any writes.
-            self.assertRaises(errors.NotLocalUrl,
-                              bzrdir.BzrDir.create_standalone_workingtree,
-                              self.get_readonly_url())
-            tree = bzrdir.BzrDir.create_standalone_workingtree('.')
-            self.assertEqual('A tree', tree)
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        # note this is deliberately readonly, as this failure should 
+        # occur before any writes.
+        self.assertRaises(errors.NotLocalUrl,
+                          bzrdir.BzrDir.create_standalone_workingtree,
+                          self.get_readonly_url(), format=format)
+        tree = bzrdir.BzrDir.create_standalone_workingtree('.', 
+                                                           format=format)
+        self.assertEqual('A tree', tree)
 
     def test_create_standalone_working_tree_under_shared_repo(self):
         # create standalone working tree always makes a repo.
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            self.make_repository('.', shared=True)
-            # note this is deliberately readonly, as this failure should 
-            # occur before any writes.
-            self.assertRaises(errors.NotLocalUrl,
-                              bzrdir.BzrDir.create_standalone_workingtree,
-                              self.get_readonly_url('child'))
-            tree = bzrdir.BzrDir.create_standalone_workingtree('child')
-            tree.bzrdir.open_repository()
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        self.make_repository('.', shared=True, format=format)
+        # note this is deliberately readonly, as this failure should 
+        # occur before any writes.
+        self.assertRaises(errors.NotLocalUrl,
+                          bzrdir.BzrDir.create_standalone_workingtree,
+                          self.get_readonly_url('child'), format=format)
+        tree = bzrdir.BzrDir.create_standalone_workingtree('child', 
+            format=format)
+        tree.bzrdir.open_repository()
 
     def test_create_branch_convenience(self):
         # outside a repo the default convenience output is a repo+branch_tree
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            branch = bzrdir.BzrDir.create_branch_convenience('.')
-            branch.bzrdir.open_workingtree()
-            branch.bzrdir.open_repository()
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        branch = bzrdir.BzrDir.create_branch_convenience('.', format=format)
+        branch.bzrdir.open_workingtree()
+        branch.bzrdir.open_repository()
 
     def test_create_branch_convenience_root(self):
         """Creating a branch at the root of a fs should work."""
         self.transport_server = MemoryServer
         # outside a repo the default convenience output is a repo+branch_tree
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            branch = bzrdir.BzrDir.create_branch_convenience(self.get_url())
-            self.assertRaises(errors.NoWorkingTree,
-                              branch.bzrdir.open_workingtree)
-            branch.bzrdir.open_repository()
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        branch = bzrdir.BzrDir.create_branch_convenience(self.get_url(), 
+                                                         format=format)
+        self.assertRaises(errors.NoWorkingTree,
+                          branch.bzrdir.open_workingtree)
+        branch.bzrdir.open_repository()
 
     def test_create_branch_convenience_under_shared_repo(self):
         # inside a repo the default convenience output is a branch+ follow the
         # repo tree policy
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            self.make_repository('.', shared=True)
-            branch = bzrdir.BzrDir.create_branch_convenience('child')
-            branch.bzrdir.open_workingtree()
-            self.assertRaises(errors.NoRepositoryPresent,
-                              branch.bzrdir.open_repository)
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        self.make_repository('.', shared=True, format=format)
+        branch = bzrdir.BzrDir.create_branch_convenience('child',
+            format=format)
+        branch.bzrdir.open_workingtree()
+        self.assertRaises(errors.NoRepositoryPresent,
+                          branch.bzrdir.open_repository)
             
     def test_create_branch_convenience_under_shared_repo_force_no_tree(self):
         # inside a repo the default convenience output is a branch+ follow the
         # repo tree policy but we can override that
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            self.make_repository('.', shared=True)
-            branch = bzrdir.BzrDir.create_branch_convenience('child',
-                force_new_tree=False)
-            self.assertRaises(errors.NoWorkingTree,
-                              branch.bzrdir.open_workingtree)
-            self.assertRaises(errors.NoRepositoryPresent,
-                              branch.bzrdir.open_repository)
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        self.make_repository('.', shared=True, format=format)
+        branch = bzrdir.BzrDir.create_branch_convenience('child',
+            force_new_tree=False, format=format)
+        self.assertRaises(errors.NoWorkingTree,
+                          branch.bzrdir.open_workingtree)
+        self.assertRaises(errors.NoRepositoryPresent,
+                          branch.bzrdir.open_repository)
             
     def test_create_branch_convenience_under_shared_repo_no_tree_policy(self):
         # inside a repo the default convenience output is a branch+ follow the
         # repo tree policy
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            repo = self.make_repository('.', shared=True)
-            repo.set_make_working_trees(False)
-            branch = bzrdir.BzrDir.create_branch_convenience('child')
-            self.assertRaises(errors.NoWorkingTree,
-                              branch.bzrdir.open_workingtree)
-            self.assertRaises(errors.NoRepositoryPresent,
-                              branch.bzrdir.open_repository)
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        repo = self.make_repository('.', shared=True, format=format)
+        repo.set_make_working_trees(False)
+        branch = bzrdir.BzrDir.create_branch_convenience('child', 
+                                                         format=format)
+        self.assertRaises(errors.NoWorkingTree,
+                          branch.bzrdir.open_workingtree)
+        self.assertRaises(errors.NoRepositoryPresent,
+                          branch.bzrdir.open_repository)
 
     def test_create_branch_convenience_under_shared_repo_no_tree_policy_force_tree(self):
         # inside a repo the default convenience output is a branch+ follow the
         # repo tree policy but we can override that
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            repo = self.make_repository('.', shared=True)
-            repo.set_make_working_trees(False)
-            branch = bzrdir.BzrDir.create_branch_convenience('child',
-                force_new_tree=True)
-            branch.bzrdir.open_workingtree()
-            self.assertRaises(errors.NoRepositoryPresent,
-                              branch.bzrdir.open_repository)
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        repo = self.make_repository('.', shared=True, format=format)
+        repo.set_make_working_trees(False)
+        branch = bzrdir.BzrDir.create_branch_convenience('child',
+            force_new_tree=True, format=format)
+        branch.bzrdir.open_workingtree()
+        self.assertRaises(errors.NoRepositoryPresent,
+                          branch.bzrdir.open_repository)
 
     def test_create_branch_convenience_under_shared_repo_force_new_repo(self):
         # inside a repo the default convenience output is overridable to give
         # repo+branch+tree
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            self.make_repository('.', shared=True)
-            branch = bzrdir.BzrDir.create_branch_convenience('child',
-                force_new_repo=True)
-            branch.bzrdir.open_repository()
-            branch.bzrdir.open_workingtree()
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        self.make_repository('.', shared=True, format=format)
+        branch = bzrdir.BzrDir.create_branch_convenience('child',
+            force_new_repo=True, format=format)
+        branch.bzrdir.open_repository()
+        branch.bzrdir.open_workingtree()
 
 
 class ChrootedTests(TestCaseWithTransport):
@@ -494,12 +522,12 @@
         # format 5 dirs need a conversion if they are not the default.
         # and they start of not the default.
         old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirFormat5())
+        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         try:
             dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
             self.assertFalse(dir.needs_format_conversion())
         finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+            bzrdir.BzrDirFormat._set_default_format(old_format)
         self.assertTrue(dir.needs_format_conversion())
 
 
@@ -529,12 +557,12 @@
     def test_needs_conversion(self):
         # format 6 dirs need an conversion if they are not the default.
         old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirMetaFormat1())
         try:
             dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
             self.assertTrue(dir.needs_format_conversion())
         finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+            bzrdir.BzrDirFormat._set_default_format(old_format)
 
 
 class NotBzrDir(bzrlib.bzrdir.BzrDir):
@@ -611,38 +639,29 @@
     
     def test_create_branch_convenience(self):
         # outside a repo the default convenience output is a repo+branch_tree
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            branch = bzrdir.BzrDir.create_branch_convenience(self.get_url('foo'))
-            self.assertRaises(errors.NoWorkingTree,
-                              branch.bzrdir.open_workingtree)
-            branch.bzrdir.open_repository()
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        branch = bzrdir.BzrDir.create_branch_convenience(
+            self.get_url('foo'), format=format)
+        self.assertRaises(errors.NoWorkingTree,
+                          branch.bzrdir.open_workingtree)
+        branch.bzrdir.open_repository()
 
     def test_create_branch_convenience_force_tree_not_local_fails(self):
         # outside a repo the default convenience output is a repo+branch_tree
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            self.assertRaises(errors.NotLocalUrl,
-                bzrdir.BzrDir.create_branch_convenience,
-                self.get_url('foo'),
-                force_new_tree=True)
-            t = get_transport(self.get_url('.'))
-            self.assertFalse(t.has('foo'))
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        self.assertRaises(errors.NotLocalUrl,
+            bzrdir.BzrDir.create_branch_convenience,
+            self.get_url('foo'),
+            force_new_tree=True,
+            format=format)
+        t = get_transport(self.get_url('.'))
+        self.assertFalse(t.has('foo'))
 
     def test_clone(self):
         # clone into a nonlocal path works
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            branch = bzrdir.BzrDir.create_branch_convenience('local')
-        finally:
-            bzrdir.BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('knit')
+        branch = bzrdir.BzrDir.create_branch_convenience('local',
+                                                         format=format)
         branch.bzrdir.open_workingtree()
         result = branch.bzrdir.clone(self.get_url('remote'))
         self.assertRaises(errors.NoWorkingTree,

=== added file 'bzrlib/tests/test_generate_docs.py'
--- bzrlib/tests/test_generate_docs.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_generate_docs.py	2007-01-02 12:40:19 +0000
@@ -0,0 +1,50 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for generating docs (man pages).
+
+This test checks that generation will be successful
+and produce non-empty output.
+"""
+
+from cStringIO import StringIO
+
+from bzrlib.tests import TestCase
+
+
+class Options:
+    """Simply container"""
+    pass
+
+
+class TestGenerateDocs(TestCase):
+
+    def setUp(self):
+        self.sio = StringIO()
+        self.options = Options()
+        self.options.bzr_name = 'bzr'
+
+    def test_man_page(self):
+        from tools.doc_generate import autodoc_man
+
+        autodoc_man.infogen(self.options, self.sio)
+        self.assertNotEqual('', self.sio.getvalue())
+
+    def test_rstx_man(self):
+        from tools.doc_generate import autodoc_rstx
+
+        autodoc_rstx.infogen(self.options, self.sio)
+        self.assertNotEqual('', self.sio.getvalue())

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-12-12 22:40:01 +0000
+++ bzrlib/tests/test_inv.py	2006-12-22 17:11:36 +0000
@@ -139,6 +139,51 @@
             ('src/sub/a', 'a-id'),
             ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir()])
             
+        self.assertEqual([
+            ('', ROOT_ID),
+            ('Makefile', 'makefile-id'),
+            ('doc', 'doc-id'),
+            ('src', 'src-id'),
+            ('zz', 'zz-id'),
+            ('src/bye.c', 'bye-id'),
+            ('src/hello.c', 'hello-id'),
+            ('src/sub', 'sub-id'),
+            ('src/zz.c', 'zzc-id'),
+            ('src/sub/a', 'a-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('a-id', 'zzc-id', 'doc-id', ROOT_ID,
+                'hello-id', 'bye-id', 'zz-id', 'src-id', 'makefile-id', 
+                'sub-id'))])
+
+        self.assertEqual([
+            ('Makefile', 'makefile-id'),
+            ('doc', 'doc-id'),
+            ('zz', 'zz-id'),
+            ('src/bye.c', 'bye-id'),
+            ('src/hello.c', 'hello-id'),
+            ('src/zz.c', 'zzc-id'),
+            ('src/sub/a', 'a-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('a-id', 'zzc-id', 'doc-id',
+                'hello-id', 'bye-id', 'zz-id', 'makefile-id'))])
+
+        self.assertEqual([
+            ('Makefile', 'makefile-id'),
+            ('src/bye.c', 'bye-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('bye-id', 'makefile-id'))])
+
+        self.assertEqual([
+            ('Makefile', 'makefile-id'),
+            ('src/bye.c', 'bye-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('bye-id', 'makefile-id'))])
+
+        self.assertEqual([
+            ('src/bye.c', 'bye-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('bye-id',))])
+
     def test_version(self):
         """Inventory remembers the text's version."""
         inv = Inventory()

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2006-12-15 08:36:50 +0000
+++ bzrlib/tests/test_options.py	2007-01-12 03:52:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,12 +16,16 @@
 
 from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
 from bzrlib.commands import Command, parse_args
-from bzrlib import errors
-from bzrlib import option
+from bzrlib import (
+        errors,
+        option,
+        symbol_versioning,
+        )
 from bzrlib.tests import TestCase
 
-# TODO: might be nice to just parse them into a structured form and test
-# against that, rather than running the whole command.
+def parse(options, args):
+    parser = option.get_optparser(dict((o.name, o) for o in options))
+    return parser.parse_args(args)
 
 class OptionTests(TestCase):
     """Command-line option tests"""
@@ -65,18 +69,48 @@
 
     def test_get_short_name(self):
         file_opt = option.Option.OPTIONS['file']
-        self.assertEquals(file_opt.short_name, 'F')
+        self.assertEquals(file_opt.short_name(), 'F')
         force_opt = option.Option.OPTIONS['force']
-        self.assertEquals(force_opt.short_name, None)
+        self.assertEquals(force_opt.short_name(), None)
+
+    def test_set_short_name(self):
+        o = option.Option('wiggle')
+        o.set_short_name('w')
+        self.assertEqual(o.short_name(), 'w')
+
+    def test_old_short_names(self):
+        # test the deprecated method for getting and setting short option
+        # names
+        expected_warning = (
+            "access to SHORT_OPTIONS was deprecated in version 0.14."
+            " Set the short option name when constructing the Option.",
+            DeprecationWarning, 2)
+        _warnings = []
+        def capture_warning(message, category, stacklevel=None):
+            _warnings.append((message, category, stacklevel))
+        old_warning_method = symbol_versioning.warn
+        try:
+            # an example of the kind of thing plugins might want to do through
+            # the old interface - make a new option and then give it a short
+            # name.
+            symbol_versioning.set_warning_method(capture_warning)
+            example_opt = option.Option('example', help='example option')
+            option.Option.SHORT_OPTIONS['w'] = example_opt
+            self.assertEqual(example_opt.short_name(), 'w')
+            self.assertEqual([expected_warning], _warnings)
+            # now check that it can actually be parsed with the registered
+            # value
+            opts, args = parse([example_opt], ['-w', 'foo'])
+            self.assertEqual(opts.example, True)
+            self.assertEqual(args, ['foo'])
+        finally:
+            symbol_versioning.set_warning_method(old_warning_method)
 
     def test_allow_dash(self):
         """Test that we can pass a plain '-' as an argument."""
         self.assertEqual((['-'], {}), parse_args(cmd_commit(), ['-']))
 
     def test_conversion(self):
-        def parse(options, args):
-            parser = option.get_optparser(dict((o.name, o) for o in options))
-            return parser.parse_args(args)
         options = [option.Option('hello')]
         opts, args = parse(options, ['--no-hello', '--hello'])
         self.assertEqual(True, opts.hello)

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-11-02 07:29:02 +0000
+++ bzrlib/tests/test_osutils.py	2007-01-04 23:36:44 +0000
@@ -580,42 +580,8 @@
             self.assertEqual([('source/lnk', 'target/lnk')], processed_links)
 
 
-class TestTerminalEncoding(TestCase):
-    """Test the auto-detection of proper terminal encoding."""
-
-    def setUp(self):
-        self._stdout = sys.stdout
-        self._stderr = sys.stderr
-        self._stdin = sys.stdin
-        self._user_encoding = bzrlib.user_encoding
-
-        self.addCleanup(self._reset)
-
-        sys.stdout = StringIOWrapper()
-        sys.stdout.encoding = 'stdout_encoding'
-        sys.stderr = StringIOWrapper()
-        sys.stderr.encoding = 'stderr_encoding'
-        sys.stdin = StringIOWrapper()
-        sys.stdin.encoding = 'stdin_encoding'
-        bzrlib.user_encoding = 'user_encoding'
-
-    def _reset(self):
-        sys.stdout = self._stdout
-        sys.stderr = self._stderr
-        sys.stdin = self._stdin
-        bzrlib.user_encoding = self._user_encoding
-
-    def test_get_terminal_encoding(self):
-        # first preference is stdout encoding
-        self.assertEqual('stdout_encoding', osutils.get_terminal_encoding())
-
-        sys.stdout.encoding = None
-        # if sys.stdout is None, fall back to sys.stdin
-        self.assertEqual('stdin_encoding', osutils.get_terminal_encoding())
-
-        sys.stdin.encoding = None
-        # and in the worst case, use bzrlib.user_encoding
-        self.assertEqual('user_encoding', osutils.get_terminal_encoding())
+#class TestTerminalEncoding has been moved to test_osutils_encodings.py
+# [bialix] 2006/12/26
 
 
 class TestSetUnsetEnv(TestCase):
@@ -677,3 +643,24 @@
         self.assertEqual(None, os.environ.get('BZR_TEST_ENV_VAR'))
         self.failIf('BZR_TEST_ENV_VAR' in os.environ)
 
+
+class TestLocalTimeOffset(TestCase):
+
+    def test_local_time_offset(self):
+        """Test that local_time_offset() returns a sane value."""
+        offset = osutils.local_time_offset()
+        self.assertTrue(isinstance(offset, int))
+        # Test that the offset is no more than a eighteen hours in
+        # either direction.
+        # Time zone handling is system specific, so it is difficult to
+        # do more specific tests, but a value outside of this range is
+        # probably wrong.
+        eighteen_hours = 18 * 3600
+        self.assertTrue(-eighteen_hours < offset < eighteen_hours)
+
+    def test_local_time_offset_with_timestamp(self):
+        """Test that local_time_offset() works with a timestamp."""
+        offset = osutils.local_time_offset(1000000000.1234567)
+        self.assertTrue(isinstance(offset, int))
+        eighteen_hours = 18 * 3600
+        self.assertTrue(-eighteen_hours < offset < eighteen_hours)

=== added file 'bzrlib/tests/test_osutils_encodings.py'
--- bzrlib/tests/test_osutils_encodings.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_osutils_encodings.py	2007-01-04 23:36:44 +0000
@@ -0,0 +1,213 @@
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the osutils wrapper."""
+
+import codecs
+import locale
+import os
+import sys
+
+import bzrlib
+from bzrlib import (
+    errors,
+    osutils,
+    )
+from bzrlib.tests import (
+        StringIOWrapper,
+        TestCase, 
+        TestSkipped,
+        )
+
+
+class FakeCodec(object):
+    """Special class that helps testing over several non-existed encodings.
+    
+    Clients can add new encoding names, but because of how codecs is
+    implemented they cannot be removed. Be careful with naming to avoid
+    collisions between tests.
+    """
+    _registered = False
+    _enabled_encodings = set()
+
+    def add(self, encoding_name):
+        """Adding encoding name to fake.
+
+        :type   encoding_name:  lowercase plain string
+        """
+        if not self._registered:
+            codecs.register(self)
+            self._registered = True
+        if encoding_name is not None:
+            self._enabled_encodings.add(encoding_name)
+        
+    def __call__(self, encoding_name):
+        """Called indirectly by codecs module during lookup"""
+        if encoding_name in self._enabled_encodings:
+            return codecs.lookup('latin-1')
+
+
+fake_codec = FakeCodec()
+
+
+class TestFakeCodec(TestCase):
+    
+    def test_fake_codec(self):
+        self.assertRaises(LookupError, codecs.lookup, 'fake')
+
+        fake_codec.add('fake')
+        codecs.lookup('fake')
+
+
+class TestTerminalEncoding(TestCase):
+    """Test the auto-detection of proper terminal encoding."""
+
+    def setUp(self):
+        self._stdout = sys.stdout
+        self._stderr = sys.stderr
+        self._stdin = sys.stdin
+        self._user_encoding = bzrlib.user_encoding
+
+        self.addCleanup(self._reset)
+
+    def make_wrapped_streams(self, 
+                             stdout_encoding,
+                             stderr_encoding,
+                             stdin_encoding,
+                             user_encoding='user_encoding',
+                             enable_fake_encodings=True):
+        sys.stdout = StringIOWrapper()
+        sys.stdout.encoding = stdout_encoding
+        sys.stderr = StringIOWrapper()
+        sys.stderr.encoding = stderr_encoding
+        sys.stdin = StringIOWrapper()
+        sys.stdin.encoding = stdin_encoding
+        bzrlib.user_encoding = user_encoding
+        if enable_fake_encodings:
+            fake_codec.add(stdout_encoding)
+            fake_codec.add(stderr_encoding)
+            fake_codec.add(stdin_encoding)
+
+    def _reset(self):
+        sys.stdout = self._stdout
+        sys.stderr = self._stderr
+        sys.stdin = self._stdin
+        bzrlib.user_encoding = self._user_encoding
+
+    def test_get_terminal_encoding(self):
+        self.make_wrapped_streams('stdout_encoding',
+                                  'stderr_encoding',
+                                  'stdin_encoding')
+
+        # first preference is stdout encoding
+        self.assertEqual('stdout_encoding', osutils.get_terminal_encoding())
+
+        sys.stdout.encoding = None
+        # if sys.stdout is None, fall back to sys.stdin
+        self.assertEqual('stdin_encoding', osutils.get_terminal_encoding())
+
+        sys.stdin.encoding = None
+        # and in the worst case, use bzrlib.user_encoding
+        self.assertEqual('user_encoding', osutils.get_terminal_encoding())
+
+    def test_terminal_cp0(self):
+        # test cp0 encoding (Windows returns cp0 when there is no encoding)
+        self.make_wrapped_streams('cp0',
+                                  'cp0',
+                                  'cp0',
+                                  user_encoding='latin-1',
+                                  enable_fake_encodings=False)
+
+        # cp0 is invalid encoding. We should fall back to user_encoding
+        self.assertEqual('latin-1', osutils.get_terminal_encoding())
+
+        # check stderr
+        self.assertEquals('', sys.stderr.getvalue())
+
+    def test_terminal_cp_unknown(self):
+        # test against really unknown encoding
+        # catch warning at stderr
+        self.make_wrapped_streams('cp-unknown',
+                                  'cp-unknown',
+                                  'cp-unknown',
+                                  user_encoding='latin-1',
+                                  enable_fake_encodings=False)
+        
+        self.assertEqual('latin-1', osutils.get_terminal_encoding())
+
+        # check stderr
+        self.assertEquals('bzr: warning: unknown terminal encoding cp-unknown.\n'
+                          '  Using encoding latin-1 instead.\n', 
+                          sys.stderr.getvalue())
+
+
+class TestUserEncoding(TestCase):
+    """Test detection of default user encoding."""
+    
+    def setUp(self):
+        self._stderr = sys.stderr
+        self._getpreferredencoding = locale.getpreferredencoding
+        self.addCleanup(self._reset)
+        sys.stderr = StringIOWrapper()
+        # save $LANG
+        self._LANG = os.environ.get('LANG')
+
+    def _reset(self):
+        locale.getpreferredencoding = self._getpreferredencoding
+        sys.stderr = self._stderr
+        # restore $LANG
+        osutils.set_or_unset_env('LANG', self._LANG)
+
+    def test_get_user_encoding(self):
+        def f():
+            return 'user_encoding'
+
+        locale.getpreferredencoding = f
+        fake_codec.add('user_encoding')
+        self.assertEquals('user_encoding', osutils.get_user_encoding(use_cache=False))
+        self.assertEquals('', sys.stderr.getvalue())
+
+    def test_user_cp0(self):
+        def f():
+            return 'cp0'
+
+        locale.getpreferredencoding = f
+        self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
+        self.assertEquals('', sys.stderr.getvalue())
+
+    def test_user_cp_unknown(self):
+        def f():
+            return 'cp-unknown'
+
+        locale.getpreferredencoding = f
+        self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
+        self.assertEquals('bzr: warning: unknown encoding cp-unknown.'
+                          ' Continuing with ascii encoding.\n',
+                          sys.stderr.getvalue())
+
+    def test_user_locale_error(self):
+        def f():
+            raise locale.Error, 'unsupported locale'
+
+        locale.getpreferredencoding = f
+        os.environ['LANG'] = 'BOGUS'
+        self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
+        self.assertEquals('bzr: warning: unsupported locale\n'
+                          '  Could not determine what text encoding to use.\n'
+                          '  This error usually means your Python interpreter\n'
+                          '  doesn\'t support the locale set by $LANG (BOGUS)\n'
+                          '  Continuing with ascii encoding.\n',
+                          sys.stderr.getvalue())

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2006-12-21 15:15:19 +0000
+++ bzrlib/tests/test_plugins.py	2006-12-29 07:53:17 +0000
@@ -22,6 +22,7 @@
 
 import os
 from StringIO import StringIO
+import zipfile
 
 import bzrlib.plugin
 import bzrlib.plugins
@@ -196,3 +197,31 @@
             # remove the plugin 'myplug'
             if getattr(bzrlib.plugins, 'myplug', None):
                 delattr(bzrlib.plugins, 'myplug')
+
+
+class TestPluginFromZip(TestCaseInTempDir):
+
+    def make_zipped_plugin(self, zip_name, filename):
+        z = zipfile.ZipFile(zip_name, 'w')
+        z.writestr(filename, PLUGIN_TEXT)
+        z.close()
+
+    def check_plugin_load(self, zip_name, plugin_name):
+        self.assertFalse(plugin_name in dir(bzrlib.plugins),
+                         'Plugin already loaded')
+        try:
+            bzrlib.plugin.load_from_zips([zip_name])
+            self.assertTrue(plugin_name in dir(bzrlib.plugins),
+                            'Plugin is not loaded')
+        finally:
+            # unregister plugin
+            if getattr(bzrlib.plugins, plugin_name, None):
+                delattr(bzrlib.plugins, plugin_name)
+
+    def test_load_module(self):
+        self.make_zipped_plugin('./test.zip', 'ziplug.py')
+        self.check_plugin_load('./test.zip', 'ziplug')
+
+    def test_load_package(self):
+        self.make_zipped_plugin('./test.zip', 'ziplug/__init__.py')
+        self.check_plugin_load('./test.zip', 'ziplug')

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2006-10-12 14:29:32 +0000
+++ bzrlib/tests/test_repository.py	2007-01-05 06:32:57 +0000
@@ -25,6 +25,7 @@
 from stat import S_ISDIR
 from StringIO import StringIO
 
+from bzrlib import symbol_versioning
 import bzrlib
 import bzrlib.bzrdir as bzrdir
 import bzrlib.errors as errors
@@ -46,7 +47,9 @@
         private_default = repository._default_format.__class__
         old_format = repository.RepositoryFormat.get_default_format()
         self.assertTrue(isinstance(old_format, private_default))
-        repository.RepositoryFormat.set_default_format(SampleRepositoryFormat())
+        self.applyDeprecated(symbol_versioning.zero_fourteen, 
+            repository.RepositoryFormat.set_default_format, 
+            SampleRepositoryFormat())
         # creating a repository should now create an instrumented dir.
         try:
             # the default branch format is used by the meta dir format
@@ -55,7 +58,8 @@
             result = dir.create_repository()
             self.assertEqual(result, 'A bzr repository dir')
         finally:
-            repository.RepositoryFormat.set_default_format(old_format)
+            self.applyDeprecated(symbol_versioning.zero_fourteen, 
+                repository.RepositoryFormat.set_default_format, old_format)
         self.assertEqual(old_format, repository.RepositoryFormat.get_default_format())
 
 

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-12-19 06:39:32 +0000
+++ bzrlib/tests/test_selftest.py	2007-01-11 08:45:51 +0000
@@ -467,7 +467,9 @@
         a branch and checking no directory was created at its relpath.
         """
         tree = self.make_branch_and_memory_tree('dir')
-        self.failIfExists('dir')
+        # Guard against regression into MemoryTransport leaking
+        # files to disk instead of keeping them in memory.
+        self.failIf(osutils.lexists('dir'))
         self.assertIsInstance(tree, memorytree.MemoryTree)
 
     def test_make_branch_and_memory_tree_with_format(self):
@@ -475,7 +477,9 @@
         format = bzrdir.BzrDirMetaFormat1()
         format.repository_format = repository.RepositoryFormat7()
         tree = self.make_branch_and_memory_tree('dir', format=format)
-        self.failIfExists('dir')
+        # Guard against regression into MemoryTransport leaking
+        # files to disk instead of keeping them in memory.
+        self.failIf(osutils.lexists('dir'))
         self.assertIsInstance(tree, memorytree.MemoryTree)
         self.assertEqual(format.repository_format.__class__,
             tree.branch.repository._format.__class__)

=== modified file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2007-01-11 08:08:21 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,14 @@
     'a_deprecated_list', ['one'], extra="Don't use me")
 
 
+a_deprecated_dict = symbol_versioning.DeprecatedDict(
+    symbol_versioning.zero_fourteen,
+    'a_deprecated_dict',
+    dict(a=42),
+    advice='Pull the other one!',
+    )
+
+
 class TestDeprecationWarnings(TestCase):
 
     def capture_warning(self, message, category, stacklevel=None):
@@ -81,8 +89,6 @@
         expected_warning = (
             "Modifying a_deprecated_list was deprecated in version 0.9."
             " Don't use me", DeprecationWarning, 3)
-        expected_doctstring = ('appending to a_deprecated_list is deprecated')
-
         old_warning_method = symbol_versioning.warn
         try:
             symbol_versioning.set_warning_method(self.capture_warning)
@@ -117,6 +123,26 @@
         finally:
             symbol_versioning.set_warning_method(old_warning_method)
 
+    def test_deprecated_dict(self):
+        expected_warning = (
+            "access to a_deprecated_dict was deprecated in version 0.14."
+            " Pull the other one!", DeprecationWarning, 2)
+        old_warning_method = symbol_versioning.warn
+        try:
+            symbol_versioning.set_warning_method(self.capture_warning)
+            self.assertEqual(len(a_deprecated_dict), 1)
+            self.assertEqual([expected_warning], self._warnings)
+
+            a_deprecated_dict['b'] = 42
+            self.assertEqual(a_deprecated_dict['b'], 42)
+            self.assertTrue('b' in a_deprecated_dict)
+            del a_deprecated_dict['b']
+            self.assertFalse('b' in a_deprecated_dict)
+            self.assertEqual([expected_warning] * 6, self._warnings)
+        finally:
+            symbol_versioning.set_warning_method(old_warning_method)
+
+
     def check_deprecated_callable(self, expected_warning, expected_docstring,
                                   expected_name, expected_module,
                                   deprecated_callable):

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-12-17 17:42:43 +0000
+++ bzrlib/tree.py	2007-01-11 15:57:49 +0000
@@ -141,7 +141,7 @@
         """
         return self.bzrdir.is_control_filename(filename)
 
-    def iter_entries_by_dir(self):
+    def iter_entries_by_dir(self, specific_file_ids=None):
         """Walk the tree in 'by_dir' order.
 
         This will yield each entry in the tree as a (path, entry) tuple. The
@@ -149,7 +149,8 @@
         preceeded by the parent of a directory, and all the contents of a 
         directory are grouped together.
         """
-        return self.inventory.iter_entries_by_dir()
+        return self.inventory.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids)
 
     def kind(self, file_id):
         raise NotImplementedError("subclasses must implement kind")
@@ -476,21 +477,16 @@
         Iteration is done in parent-to-child order, relative to the to_tree.
         """
         to_paths = {}
-        from_entries_by_dir = list(from_tree.inventory.iter_entries_by_dir())
+        from_entries_by_dir = list(from_tree.inventory.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids))
         from_data = dict((e.file_id, (p, e)) for p, e in from_entries_by_dir)
-        to_entries_by_dir = list(to_tree.inventory.iter_entries_by_dir())
-        if specific_file_ids is not None:
-            specific_file_ids = set(specific_file_ids)
-            num_entries = len(specific_file_ids)
-        else:
-            num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
+        to_entries_by_dir = list(to_tree.inventory.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids))
+        num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
         entry_count = 0
         for to_path, to_entry in to_entries_by_dir:
             file_id = to_entry.file_id
             to_paths[file_id] = to_path
-            if (specific_file_ids is not None and 
-                file_id not in specific_file_ids):
-                continue
             entry_count += 1
             changed_content = False
             from_path, from_entry = from_data.get(file_id, (None, None))
@@ -501,8 +497,7 @@
                 from_parent = from_entry.parent_id
                 from_kind, from_executable, from_stat = \
                     from_tree._comparison_data(from_entry, from_path)
-                if specific_file_ids is None:
-                    entry_count += 1
+                entry_count += 1
             else:
                 from_versioned = False
                 from_kind = None
@@ -538,19 +533,22 @@
                 yield (file_id, to_path, changed_content, versioned, parent,
                        name, kind, executable)
 
-        for path, from_entry in from_entries_by_dir:
-            file_id = from_entry.file_id
-            if file_id in to_paths:
-                continue
+        def get_to_path(from_entry):
             if from_entry.parent_id is None:
                 to_path = ''
             else:
+                if from_entry.parent_id not in to_paths:
+                    get_to_path(from_tree.inventory[from_entry.parent_id])
                 to_path = osutils.pathjoin(to_paths[from_entry.parent_id],
                                            from_entry.name)
-            to_paths[file_id] = to_path
-            if (specific_file_ids is not None and 
-                file_id not in specific_file_ids):
+            to_paths[from_entry.file_id] = to_path
+            return to_path
+
+        for path, from_entry in from_entries_by_dir:
+            file_id = from_entry.file_id
+            if file_id in to_paths:
                 continue
+            to_path = get_to_path(from_entry)
             entry_count += 1
             if pb is not None:
                 pb.update('comparing files', entry_count, num_entries)

=== modified file 'doc/configuration.txt'
--- doc/configuration.txt	2006-11-11 19:20:08 +0000
+++ doc/configuration.txt	2007-01-02 07:16:27 +0000
@@ -16,14 +16,14 @@
 An ini file has three types of contructs: section headers, section
 variables and comments.
 
-comment
--------
+Comments
+--------
 A comment is any line that starts with a "#" (sometimes called a "hash
 mark", "pound sign" or "number sign"). Comment lines are ignored by
 Bazaar when parsing ini files.
 
-section header
---------------
+Section Headers
+---------------
 A section header is a word enclosed in brackets that starts at the begining
 of a line, typical section headers look like this::
 
@@ -43,7 +43,7 @@
 
 
 
-section variables
+Section Variables
 -----------------
 
 A section variable resides within a section. A section variable contains a
@@ -54,6 +54,36 @@
     check_signatures = require
 
 
+Variable Policies
+-----------------
+
+Variables defined in a section affect the named directory or URL plus
+any locations they contain.  Policies can be used to change how a
+variable value is interpreted for contained locations.  Currently
+there are three policies available:
+
+ none:
+   the value is interpreted the same for contained locations.  This is
+   the default behaviour.
+ norecurse:
+   the value is only used for the exact location specified by the
+   section name.
+ appendpath:
+   for contained locations, any additional path components are
+   appended to the value.
+
+Policies are specified by keys with names of the form "$var:policy".
+For example, to define the push location for a tree of branches, the
+following could be used::
+
+  [/top/location]
+  push_location = sftp://example.com/location
+  push_location:policy = appendpath
+
+With this configuration, the push location for ``/top/location/branch1``
+would be ``sftp://example.com/location/branch1``.
+
+
 The main configuration file, bazaar.conf
 ----------------------------------------
 

=== modified file 'setup.py'
--- setup.py	2006-09-08 18:46:29 +0000
+++ setup.py	2007-01-15 03:31:34 +0000
@@ -188,12 +188,18 @@
                                      version = version_str,
                                      description = META_INFO['description'],
                                      author = META_INFO['author'],
-                                     copyright = "(c) Canonical Ltd, 2005-2006",
+                                     copyright = "(c) Canonical Ltd, 2005-2007",
                                      company_name = "Canonical Ltd.",
                                      comments = META_INFO['description'],
                                     )
+
+    additional_packages =  []
+    if sys.version.startswith('2.4'):
+        # adding elementtree package
+        additional_packages.append('elementtree')
+
     options_list = {"py2exe": {"packages": BZRLIB['packages'] +
-                                           ['elementtree'],
+                                           additional_packages,
                                "excludes": ["Tkinter", "medusa"],
                                "dist_dir": "win32_bzr.exe",
                               },

=== modified file 'tools/doc_generate/autodoc_man.py'
--- tools/doc_generate/autodoc_man.py	2006-08-16 04:04:30 +0000
+++ tools/doc_generate/autodoc_man.py	2007-01-04 01:33:38 +0000
@@ -184,7 +184,7 @@
 E-Mail address of the user. Overrides default user config.
 .TP
 .I "EMAIL"
-E-Mail address of the user. Overriddes default user config.
+E-Mail address of the user. Overrides default user config.
 .TP
 .I "BZR_EDITOR"
 Editor for editing commit messages

=== modified file 'tools/doc_generate/autodoc_rstx.py'
--- tools/doc_generate/autodoc_rstx.py	2006-08-06 23:35:09 +0000
+++ tools/doc_generate/autodoc_rstx.py	2007-01-10 07:28:57 +0000
@@ -194,7 +194,7 @@
                 E-Mail address of the user. Overrides default user config.
 
 **EMAIL**
-                E-Mail address of the user. Overriddes default user config.
+                E-Mail address of the user. Overrides default user config.
 
 **BZR_EDITOR**
                 Editor for editing commit messages

=== modified file 'tools/win32/bzr.iss.cog'
--- tools/win32/bzr.iss.cog	2006-10-14 08:51:07 +0000
+++ tools/win32/bzr.iss.cog	2007-01-13 20:00:38 +0000
@@ -7,6 +7,14 @@
 AppName=Bazaar
 
 ; [[[cog
+; # Python 2.5 compatibility code
+; import os
+; import sys
+; cwd = os.getcwd()
+; if cwd not in sys.path:
+;     sys.path.insert(0, cwd)
+; #/Python 2.5 compatibility code
+;
 ; import bzrlib
 ; try: 
 ;     VERSION = bzrlib.__version__

